<?php
require_once(_PS_MODULE_DIR_ . 'freshmail/lib/Auth.php');
class AdminFreshMailFormController extends AdminController{
    
    public static $UPDATE_SUCCESS = "Success!";
    public static $ERROR = "Error!";
    public static $POSITION = array(
        '1' => 'Left column',
        '2' => 'Right column',
        '3' => 'Footer',
        '4' => 'Top'
    );
    
    public function initContent() {
        $this->initTabModuleList();
        $this->initToolbar();
        parent::initContent();
    }
    
    public function initPageHeaderToolbar()
    {
        if (empty($this->display)) {
            $this->page_header_toolbar_btn['new_product'] = array(
                    'href' => self::$currentIndex.'&add'.$this->table.'=1&token='.$this->token,
                    'desc' => $this->l('Add new form', null, null, false),
                    'icon' => 'process-icon-new'
                );
        }
        
        parent::initPageHeaderToolbar();
    }
    
    	public function initToolbar()
	{
		switch ($this->display)
		{
                    default: // list
                    $this->toolbar_btn['new'] = array(
                        'href' => self::$currentIndex.'&add'.$this->table.'=1&token='.$this->token,
                        'desc' => $this->l('Add New Form')
                    );
		}
                
                unset($this->toolbar_btn['back']);
	}
    
    public function __construct() {
        
        Auth::check();
        
        $this->bootstrap = true;
        $this->table = 'fm_form';
        $this->tpl_folder = 'freshmail_controller';
        $this->lang = false;
        $this->explicitSelect = true;
        $this->context = Context::getContext();
        $this->show_cancel_button = false;
        
        
        // Building the list of records stored within the "article" table
        $this->fields_list = array(
            'id_fm_form' => array(
                'title' => 'ID',
                'align' => 'center',
                'width' => 25
            ),
            'name' => array(
                'title' => 'Name',
                'width' => 'auto',
            ),
            'hash_list' => array(
                'title' => 'List Hash',
                'width' => 'auto'
            ),
            'position' => array(
                'title' => 'Position',
                'width' => 'auto'
            ),
            'count_fields' => array(
                'title' => 'Count fields <sup>(include)</sup>',
                'width' => 'auto'
            ),
            'active' => array(
                'title' => 'Active',
                'width' => 'auto'
            ),
        );
  
        parent::__construct();
    }
    
    // This method generates the list of results
    public function renderList(){
        
        $api = new Api();
        $hashListSQL = $api->getAllHashListSQL();
        $hashList = $api->getAllHashList();
        $forms = Form::getAllFormsFull();
        
        foreach($forms as $k => $form) {
            if(!in_array($form['hash_list'], $hashList)) {
                $formObj = new Form($form['id_fm_form']);
                $formObj->active = 0;
                $formObj->update(true);
            }
        }

        $this->_select = 'a.`id_fm_form`, a.`hash_list`, a.`position`, a.`active`, count('._DB_PREFIX_.'fm_list_field.id) as `count_fields`, `a`.`name`';
        $this->_join = 'LEFT JOIN '._DB_PREFIX_.'fm_list_field ON a.id_fm_form = '._DB_PREFIX_.'fm_list_field.id_form AND '._DB_PREFIX_.'fm_list_field.include_field = 1';
        if(!empty($hashListSQL)) {
            $this->_where = ' AND a.`hash_list`  IN (' . $hashListSQL . ')';
        }
        $this->_group = 'GROUP BY a.id_fm_form';
        $this->_orderBy = 'id_fm_form';
        $this->_orderWay = 'DESC';
        
        // Adds an Edit button for each result
        $this->addRowAction('edit');
  
        // Adds a Delete button for each result
        $this->addRowAction('delete');
        
        
        if (!($this->fields_list && is_array($this->fields_list))) {
            return false;
        }
        $this->getList($this->context->language->id);

        foreach($this->_list as $k => $v) {
            if(isset($v['active'])) {
                if((int)$this->_list[$k]['active'] == 0) {
                    $this->_list[$k]['active'] = $this->l('Disabled');
                } else {
                    $this->_list[$k]['active'] = $this->l('Enabled');
                }
                
                if(isset($v['position']) && isset(self::$POSITION[$this->_list[$k]['position']])) {
                    $this->_list[$k]['position'] = self::$POSITION[$this->_list[$k]['position']];
                }
            }
        }
        
        // If list has 'active' field, we automatically create bulk action
        if (isset($this->fields_list) && is_array($this->fields_list) && array_key_exists('active', $this->fields_list)
            && !empty($this->fields_list['active'])) {
            if (!is_array($this->bulk_actions)) {
                $this->bulk_actions = array();
            }
        }

        $helper = new HelperList();
        
        // Empty list is ok
        if (!is_array($this->_list)) {
            $this->displayWarning($this->l('Bad SQL query', 'Helper').'<br />'.htmlspecialchars($this->_list_error));
            return false;
        }

        $this->setHelperDisplay($helper);
        $helper->simple_header = true;
        $helper->_default_pagination = $this->_default_pagination;
        $helper->_pagination = $this->_pagination;
        $helper->tpl_vars = $this->getTemplateListVars();
        
        // For compatibility reasons, we have to check standard actions in class attributes
        foreach ($this->actions_available as $action) {
            if (!in_array($action, $this->actions) && isset($this->$action) && $this->$action) {
                $this->actions[] = $action;
            }
        }
        $helper->has_value = false;
        $helper->is_cms = $this->is_cms;
        $helper->sql = $this->_listsql;
        $list = $helper->generateList($this->_list, $this->fields_list);

        return $list;
    }
    
    private function valid($action = 'add')
    {
        $this->requiredFields = array('name', 'hash_list', 'position');
        foreach($this->requiredFields as $k => $v) {
            if(!empty($v)) {
                $value = Tools::getValue($v);
                if(empty($value)) {
                    $title = ucfirst($this->fields_list[$v]['title']);
                    $label = $this->l($title . ' is required!');
                    $this->errors[] = Tools::displayError($label);
                }
            }
        }
        if(count($this->errors)) {
            $this->action = $action;
            return false;
        }
        return true;
    }


    public function postProcess()
    {   
        if(Tools::getValue('addid_fm_form')) {
            $action = 'addfm_form';
        } elseif(Tools::getValue('editid_fm_form')) {
            $action = 'updatefm_form';
        }
        
        if (Tools::isSubmit('submitFieldsList') && Tools::getValue('addid_fm_form')) {
            
            if(!$this->valid('add')) {
                return parent::postProcess();
            }
            
            try {
                
                $newForm = new Form();
                $newForm->name = Tools::getValue('name');
                $newForm->hash_list = Tools::getValue('hash_list');
                $newForm->position = Tools::getValue('position');
                $newForm->active = Tools::getValue('active');
                $newForm->save();

                $fieldsList = Tools::getValue('fields');
                if(empty($fieldsList)) {
                    $fieldsList = array();
                }
                
                foreach ($fieldsList as $hashList => $fields) {
                    $field = new Field();   

                    $field->name = $fields['name'];
                    $field->tag = $fields['tag'];
                    $field->hash = $fields['hash'];
                    $field->type = $fields['type'];
                    $field->displayname = $fields['displayname'];

                    if(isset($fields['required'])) {
                        $field->require_field =  true;
                    }
                    if(isset($fields['include'])) {
                        $field->include_field = true;
                    }

                    $field->hash_list = Tools::getValue('hash_list');
                    $field->id_form = $newForm->id;
                    
                    $id = $field->save();
                }
                $link = $this->context->link->getAdminLink('AdminFreshmailform').'&conf=4&updatefm_form=1&id_fm_form=' . $newForm->id;
                Tools::redirectAdmin($link);
            } catch (Exception $e) {
                if(count($this->errors) <= 0) {
                    $this->errors[] = Tools::displayError($e->getMessage());
                }
            }
        } elseif(Tools::isSubmit('submitFieldsList') && Tools::getValue('editid_fm_form') && Tools::getValue('id')) {
            try {
                $id = (int) Tools::getValue('id');
                $form = new Form($id);
                $form->name = Tools::getValue('name');

                $form->position = Tools::getValue('position');
                $form->active = Tools::getValue('active');
                $form->update();

                $fieldsList = Tools::getValue('fields');
                
                foreach ($fieldsList as $hashList => $fields) {
                    
                    $field = new Field($fields['id']);
                    $field->displayname = $fields['displayname'];
                    $field->require_field = (int)$fields['required'];
                    $field->include_field = (int)$fields['include'];
                    
                    $field->save();
                }
                $link = $this->context->link->getAdminLink('AdminFreshmailform', true).'&conf=4&updatefm_form=1&id_fm_form=' . $id;
                Tools::redirectAdmin($link);
            } catch (Exception $e) {
                $this->displayWarning($this->l(self::$ERROR) . ' ' . $e->getMessage());
            }
        }
        return parent::postProcess();
    }
    
    // This method generates the Add/Edit form
    public function renderForm() {
        $this->submit_action = $this->display . $this->identifier;
        $this->show_form_cancel_button = false;
        $api = new Api();
        $lists = $api->getAllList();
        
        $this->fields_form = array(
            'legend' => array(
                'title' => $this->l('Formularz i pola dodatkowe'),
            ),
                'input' => array(
                    array(
                        'type' => 'text',
                        'label' => 'Name',
                        'name' => 'name',
                        'required' => true,
                        'desc' => $this->l('Twoja nazwa formularza'),
                        'class' => 'fixed-width-xxl',
                        'hint' => $this->l('Invalid characters:').' &lt;&gt;;=#{}',
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Lists'),
                        'desc' => $this->l('Listy odbiorców zdefiniowane w aplikacji Freshmail'),
                        'name' => 'hash_list',
                        'multiple' => false,
                        'required' => true,
                        'options' => array(
                            'query' => $lists,
                            'id' => 'key',
                            'name' => 'name'
                        ),
                        "disabled" => ($this->display == 'edit') ? true : false
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Position'),
                        'name' => 'position',
                        'required' => true,
                        'options' => array(
                            'query' => array(
                                array('key' => '1', 'name' => self::$POSITION[1]),
                                array('key' => '2', 'name' => self::$POSITION[2]),
                                array('key' => '3', 'name' => self::$POSITION[3]),
                                array('key' => '4', 'name' => self::$POSITION[4]),
                            ),
                            'id' => 'key',
                            'name' => 'name'
                        )
                    ),
                    array(
                        'type'      => 'switch',
                        'label'     => $this->l('Enable form'),
                        'desc'      => $this->l('To determine the active form in an application.'),
                        'name'      => 'active',
                        'required'  => true,
                        'class'     => 'switch prestashop-switch',
                        'is_bool'   => true,
                        'values'    => array(
                            array(
                                'id'    => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id'    => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    )
                ),
                'submit' => array(
                    'name' => 'submitFieldsList',
                    'title' => $this->l('Save'),
                    'class' => 'btn btn-default pull-right'
                ),
            'buttons' => array(
                array(
                    'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminFreshmailformController'),
                    'title' => $this->l('Back to list'),
                    'icon' => 'process-icon-back'
                )
            )
            //)
        );
        
        $form = new Form((int)Tools::getValue('id_fm_form'));
        
        $this->fields_value['name'] = $form->name;
        $this->fields_value['position'] = $form->position;
        $this->fields_value['hash_list'] = $form->hash_list;
        $this->fields_value['active'] = $form->active;
        $this->fields_value['confirm'] = $form->confirm;
        $this->fields_value['subscribers_status'] = $form->subscribers_status;
        
        $fieldsDb = $form->fields;
        $fieldsApi = $api->getAllFieldsByIdHashList($form->hash_list);
        $this->refreshFields($fieldsApi, $fieldsDb, $form->hash_list, $form->id);
        $fieldsList = false;
        
        if($fieldsDb) {
            $fieldsList = $fieldsDb;
        }
        
        
        $this->context->smarty->assign(
            array(
                'fields_list'   => true,
                'fields_list_update' => ($fieldsList) ? $fieldsList : false,
                'id'            => $form->id,
                'display'       => $this->display
            )
        );
        
        return parent::renderForm();
    }
    
    public function refreshFields($fieldsApi = null, $fieldsDb = null, $hashList = null, $formId = null) {
        if((empty($fieldsApi) && empty($fieldsDb)) || !$hashList || !$formId) {
            return false;
        }
        
        $api = new Api();
        $hashKeys = $api->getAllFieldHashByHashList($hashList);
        
        $fieldModel = new Field();
        $hashListFieldDb = $fieldModel->getFieldsHashByIdForm($formId);
        $change = 0;
        $delete = 0;
        
        foreach($fieldsDb as $k => $field) {
            if($field['tag'] != 'email' && !in_array($field['hash'], array_keys($hashKeys))) {
                $fieldObj = new Field($field['id']);
                $fieldObj->delete();
                $delete = 1;
            } else if($field['tag'] != 'email' && ($hashKeys[$field['hash']]['tag'] != $field['tag'] || $hashKeys[$field['hash']]['type'] != $field['type'])) {
                $fieldObj = new Field($field['id']);
                $fieldObj->tag = $hashKeys[$field['hash']]['tag'];
                $fieldObj->type = $hashKeys[$field['hash']]['type'];
                $fieldObj->update();
                $change = 1;
            }
        }

        foreach ($fieldsApi as $k => $field) {
            if(!in_array($field['hash'], $hashListFieldDb)) {
                $fieldObj = new Field();
                foreach($field as $fieldName => $fieldValue) {
                    if(!empty($fieldValue)) {
                        $fieldObj->{$fieldName} = $fieldValue;
                    }
                }
                $fieldObj->hash_list = $hashList;
                $fieldObj->id_form = $formId;
                $fieldObj->save();
                $change = 1;
            }
        }
        
        if($change || $delete) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminFreshmailform').'&id_fm_form=' . (int) $formId. '&updatefm_form=1&conf=4');
        }
    }
    
    public function processFilter()
    {
        
    }
    
    public function displayEditLink($token = null, $id, $name = null)
    {
        $tpl = $this->createTemplate('helpers/list/list_action_edit.tpl');
        if (!array_key_exists('Edit', self::$cache_lang)) {
            self::$cache_lang['Edit'] = $this->l('Edit', 'Helper');
        }

        $href = self::$currentIndex.'&'.$this->identifier.'='.$id.'&update'.$this->table.'&token='.($token != null ? $token : $this->token);

        if ($this->display == 'view') {
            $href = Context::getContext()->link->getAdminLink('AdminCustomers').'&id_customer='.(int)$id.'&updatecustomer';
        }

        $tpl->assign(array(
            'href' => $href,
            'action' => self::$cache_lang['Edit'],
            'id' => $id
        ));

        return $tpl->fetch();
    }
    
    public function checkAccess()
    {
        return true;
    }

    public function viewAccess()
    {
        return true;
    }
    
    public function displayAjax() {
        $return = array(
            'hasError' => true,
            'errors' => 'Error'
        );
        die(Tools::jsonEncode($return));
    }
    
    public function setMedia()
    {   
        $this->addJS(
            array(
                _PS_MODULE_DIR_.'freshmail/views/js/updateFieldsForm.js'
            )
        );
        
        return parent::setMedia();
    }
    
    public function displayAjaxUpdateFieldsForm() 
    {
        $this->ajax = true;
        $hash = Tools::getValue('hash');
        $api = new Api();
        $fieldsApi = $api->getAllFieldsByIdHashList($hash);
        
        $fieldsApi[] = array(
            'displayname' => "E-mail",
            'hash_list' => $hash,
            "hash" => $hash,
            "name" => "E-mail",
            "tag" => "email"
        );
        
        $result = array(
            'status' => 'OK',
            'fields' => $fieldsApi
        );
        die(Tools::jsonEncode($result));
    }
    
    public function processDelete()
    {
        $id = (int)Tools::getValue('id_fm_form');
        
        $form = new Form($id);
        $form->delete();
        Tools::redirectAdmin($this->context->link->getAdminLink('AdminFreshmailform').'&conf=1');
    }
    
    public function processAdd()
    {
        $this->display = 'add';
    }
}
?>
