<?php
class FreshMailApiController extends AdminController
{
    const INTEGRATION_CREATED = 'FreshMail integration for Prestashop created successfully!';
    const SUCCESS_UPDATE_API_KEYS = 'API keys update successfully!';
    const FRESHMAIL_URL = 'http://freshmail.com';
    
    public function __construct()
    {
        $this->bootstrap = true;
        $this->lang = true;
        $this->deleted = false;
        $this->colorOnBackground = false;
        $this->bulk_actions = array('delete' => array(
            'text' => $this->l('Delete selected'), 
            'confirm' => $this->l('Delete selected items?'))
        );
        $this->context = Context::getContext();
        parent::__construct();
    }
    
    public function initContent()
    {
        parent::initContent();
        $this->initTabModuleList();
        $this->initToolbar();
        $this->initPageHeaderToolbar();
        $this->addToolBarModulesListButton();
        
        self::checkKeys($this->context);
        
        unset($this->toolbar_btn['save']);
        $back = $this->context->link->getAdminLink('AdminDashboard');
        $this->toolbar_btn['back'] = array(
            'href' => $back,
            'desc' => $this->l('Back to the dashboard'),
        );
        
        $helpArray = array(
            'url_post'                  => self::$currentIndex . '&token=' . $this->token,
            'module_templates'          => _PS_MODULE_DIR_.'freshmail/views/templates/admin/',
            'show_page_header_toolbar'  => $this->show_page_header_toolbar,
            'page_header_toolbar_title' => $this->page_header_toolbar_title,
            'page_header_toolbar_btn'   => $this->page_header_toolbar_btn,
            'response'                  => null,
        );
        
        if (Tools::isSubmit('submitEnterApiKeys')) {
            $key = Tools::getValue('key');
            $secretKey = Tools::getValue('secret_key');
            
            $keyUpdateStatus = Configuration::updateValue('FRESHMAIL_API_KEY', $key);
            $keySecretUpdateStatus = Configuration::updateValue('FRESHMAIL_API_SECRECT_KEY', $secretKey);
            if($keyUpdateStatus && $keySecretUpdateStatus) {
                $link = $this->context->link->getAdminLink('FreshMailApi').'&conf=6';
                Tools::redirectAdmin($link);
            } else {
                Tools::displayError('An error occurred while updating.');
            }
        }elseif(Tools::isSubmit('submitConnectToApi')) {
            $this->checkConnection();
            
            $response = $this->context->smarty->getTemplateVars('response');
            if($response && $response['status'] == 'OK') {
                $helpArray['success'] = self::INTEGRATION_CREATED;
            }
        }
        
        $helpArray['FRESHMAIL_API_KEY'] = Configuration::get('FRESHMAIL_API_KEY');
        $helpArray['FRESHMAIL_API_SECRECT_KEY'] = Configuration::get('FRESHMAIL_API_SECRECT_KEY');
        if(!empty($helpArray['FRESHMAIL_API_KEY']) && !empty($helpArray['FRESHMAIL_API_SECRECT_KEY'])) {
            $helpArray['showCheck'] = true;
        } else {
            $helpArray['showCheck'] = false;
        }
        $this->context->smarty->assign($helpArray);
        
        $this->setTemplate('api.tpl');
    }
    
    public function createTemplate($tpl_name)
    {
        $path = dirname(__FILE__) . DS . '..' . DS . '..' . DS . 'views' . DS . 'templates' . DS . 'admin' . DS;

        return $this->context->smarty->createTemplate($path . $tpl_name, $this->context->smarty);
    }

    public function checkAccess()
    {
        return true;
    }

    public function viewAccess()
    {
        return true;
    }
    
    public static function checkKeys($context = null)
    {
        if(!$context) {
            return false;
        }
        
        $key = Configuration::get('FRESHMAIL_API_KEY');
        $keySecrect = Configuration::get('FRESHMAIL_API_SECRECT_KEY');
        if(empty($key) || empty($keySecrect)) {
            $context->smarty->assign('errorEmptyKeys', true);
            $context->smarty->assign('errorEmptyKeysUrl', self::FRESHMAIL_URL);
        }
    }
    
    public function checkConnection()
    {
        $rest = new FmRestAPI();
        $rest->setApiKey(Configuration::get('FRESHMAIL_API_KEY'));
        $rest->setApiSecret(Configuration::get('FRESHMAIL_API_SECRECT_KEY'));
        
        try {
            $response = $rest->doRequest('ping');
        } catch (Exception $e) {
            $response['status'] = 'Error message: '.$e->getMessage().', Error code: '.$e->getCode().', HTTP code: '.$rest->getHttpCode();
        }
        
        $this->context->smarty->assign(array('response' => $response));
    }
}