<?php
require_once(_PS_MODULE_DIR_ . 'freshmail/lib/Auth.php');
require_once(_PS_MODULE_DIR_ . 'freshmail/lib/Api.php');
require_once(_PS_MODULE_DIR_ . 'freshmail/models/Form.php');
require_once(_PS_MODULE_DIR_ . 'freshmail/classes/FormRepository.php');
require_once(_PS_MODULE_DIR_ . 'freshmail/models/Field.php');

class Freshmail extends Module
{
        static $LEFT_COLUMN = 1;
        static $RIGHT_COLUMN = 2;
        static $FOOTER = 3;
        static $TOP = 4;
        static $SUBSCRIBER_STATUS_NO_ACTIVE = 2;
        static $CONFIRM_NO_ACTIVE = 1;
    
    public function __construct()
    {
        $this->name = 'freshmail';
        $this->tab = 'advertising_marketing';
        $this->version = '0.5';
        $this->author = 'FreshMail';
        $this->need_instance = 0;
        
        $this->controllers = array('details');
        parent::__construct();
        $this->bootstrap = true;
        $this->ps_versions_compliancy = array('min' => '1.6.1.1', 'max' => _PS_VERSION_);
        $this->displayName = $this->l('FreshMail for Presta');
        $this->description = $this->l('Synchronizes your newsletter subscribers and shop items with Freshmail.com');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');
    }
    
    public function installFm() {
        @copy(_PS_MODULE_DIR_.$this->name.'/views/img/fm.png', _PS_IMG_DIR_.'t/fm.png');
        
        $sqlForm = "CREATE TABLE IF NOT EXISTS `" . _DB_PREFIX_ . "fm_form` (
            `id_fm_form` int(11) NOT NULL AUTO_INCREMENT,
            `hash_list` char(10) COLLATE utf8_general_ci NOT NULL,
            `name` varchar(255) COLLATE utf8_general_ci NOT NULL,
            `count` int(11) NOT NULL,
            `position` int(11) NOT NULL,
            `last` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
            `active` tinyint(4) NOT NULL,
            `subscribers_status` int(1) NOT NULL DEFAULT '1',
            `confirm` int(11) NOT NULL DEFAULT '0',
            PRIMARY KEY (`id_fm_form`),
            KEY `id_fm_form` (`hash_list`) USING BTREE
          ) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;";
        
        Db::getInstance()->Execute($sqlForm);
        
        $sqlField = "CREATE TABLE IF NOT EXISTS `" . _DB_PREFIX_ . "fm_list_field` (
            `id` int(11) NOT NULL AUTO_INCREMENT,
            `tag` varchar(255) COLLATE utf8_general_ci NOT NULL,
            `name` varchar(255) COLLATE utf8_general_ci NOT NULL,
            `require_field` tinyint(4) NOT NULL,
            `include_field` tinyint(4) NOT NULL,
            `hash_list` char(10) COLLATE utf8_general_ci NOT NULL,
            `hash` varchar(11) COLLATE utf8_general_ci NOT NULL,
            `displayname` varchar(255) COLLATE utf8_general_ci NOT NULL,
            `id_form` int(11) NOT NULL,
            `type` varchar(11) COLLATE utf8_general_ci NOT NULL,
            PRIMARY KEY (`id`),
            KEY `list_id` (`hash_list`),
            CONSTRAINT `"._DB_PREFIX_."fm_list_field_ibfk_1` FOREIGN KEY (`hash_list`) REFERENCES `"._DB_PREFIX_."fm_form` (`hash_list`) ON DELETE CASCADE ON UPDATE CASCADE
          ) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;";
        Db::getInstance()->Execute($sqlField);
        
        $this->override_file();
        
        Configuration::updateValue('FRESHMAIL_SUBMISSION_SUCCESS_MESSAGE', $this->l('Your sign up request was successful! Please check your email inbox.'));
        Configuration::updateValue('FRESHMAIL_SUBMISSION_FAILURE_MESSAGE', $this->l('Oops. Something went wrong. Please try again later.'));
        Configuration::updateValue('FRESHMAIL_ALREADY_SUBSCRIBED_MESSAGE', $this->l('Given email address is already subscribed, thank you!'));
        Configuration::updateValue('FRESHMAIL_INVALID_EMAIL_ADDRESS_MESSAGE', $this->l('Please provide a valid email address'));
        Configuration::updateValue('FRESHMAIL_REQUIRED_FIELD_MISSING_MESSAGE', $this->l('Please fill all the required fields'));
        Configuration::updateValue('FRESHMAIL_FORM_SUBSCRIBE_BUTTON', $this->l('Sign me up!'));
        Configuration::updateValue('FRESHMAIL_SIGNUP_LABEL', $this->l('Sign me up for the newsletter'));
        Configuration::updateValue('FRESHMAIL_ACCEPT_EMAILS_PARTNERS_LABEL', $this->l('I agree to receive email communications from partners'));
        
    }
    
    public function override_file() {
        $dir = _PS_ROOT_DIR_."/override/controllers/admin/templates/fresh_mail_form/helpers/form/";
        $file = 'form.tpl';

        if(!file_exists($dir) && !is_dir($dir)) {
            @mkdir($dir, 0777, true);
        }
        $fileData = file_get_contents(dirname(__FILE__).'/tmp/' . $file);
        $filepath = $dir . $file;
        file_put_contents($filepath,$fileData);
    }
    
    public function install()
    {
        $this->installFm();
        
        if (!parent::install() ||
            !$this->registerHook('header') ||
            !$this->registerHook('displayAdminProductsExtra') ||
            !$this->registerHook('displayHeader') ||
            !$this->registerHook('displayTop') ||
            !$this->registerHook('displayLeftColumn') ||
            !$this->registerHook('displayRightColumn') ||
            !$this->registerHook('displayFooter') ||
            !$this->registerHook('backOfficeHeader') ||
            !$this->registerHook('displayCustomerAccountForm') ||
            !$this->registerHook('actionCustomerAccountAdd')
            ) {
                return false;
            }
        
        // Install Tabs
        $parent_tab = new Tab();
        $parent_tab->name[(int)Configuration::get('PS_LANG_DEFAULT')] = $this->l('FreshMail');
        $parent_tab->class_name = 'FreshMailTab';
        $parent_tab->id_parent = 0; // Home tab
        $parent_tab->module = $this->name;
        $parent_tab->add();
        
        $tabForms = new Tab();
        $tabForms->name[$this->context->language->id] = $this->l('New Form');
        $tabForms->class_name = 'AdminFreshmailform';
        $tabForms->id_parent = $parent_tab->id;
        $tabForms->module = $this->name;
        $tabForms->add();
        
        $tabApi = new Tab();
        $tabApi->name[$this->context->language->id] = $this->l('Login');
        $tabApi->class_name = 'FreshMailApi';
        $tabApi->id_parent = $parent_tab->id;
        $tabApi->module = $this->name;
        $tabApi->add();
        
        $tabNewForm = new Tab();
        $tabNewForm->name[$this->context->language->id] = $this->l('Forms');
        $tabNewForm->class_name = 'AdminFreshmailform';
        $tabNewForm->id_parent = $parent_tab->id;
        $tabNewForm->module = $this->name;
        $tabNewForm->add();
        
        $tabCheckboxes = new Tab();
        $tabCheckboxes->name[$this->context->language->id] = $this->l('Checkboxes');
        $tabCheckboxes->class_name = 'AdminFreshmailcheckboxes';
        $tabCheckboxes->id_parent = $parent_tab->id;
        $tabCheckboxes->module = $this->name;
        $tabCheckboxes->add();
        
        $tabMessages = new Tab();
        $tabMessages->name[$this->context->language->id] = $this->l('Messages');
        $tabMessages->class_name = 'AdminFreshmailMessages';
        $tabMessages->id_parent = $parent_tab->id;
        $tabMessages->module = $this->name;
        $tabMessages->add();
        
        return true;
    }
    
    
    public function uninstall()
    {
        $tab = new Tab((int)Tab::getIdFromClassName('FreshMailTab'));
        $tab->delete();
        
        $sqlDropFields = 'DROP TABLE ' . _DB_PREFIX_ . 'fm_list_field;';
        Db::getInstance()->Execute($sqlDropFields);
        
        $sqlDropFields = 'DROP TABLE ' . _DB_PREFIX_ . 'fm_form;';
        Db::getInstance()->Execute($sqlDropFields);
        
        $sqlDropFields = 'DELETE FROM ' . _DB_PREFIX_ . 'configuration WHERE name LIKE "%FRESHMAIL%" ';
        Db::getInstance()->Execute($sqlDropFields);
         
        return parent::uninstall();
    }
    
    public function hookBackOfficeHeader()
    {
        $this->context->controller->addJquery();
        $this->context->controller->addJS($this->_path . 'views/js/leftMenu.js');
        $this->context->controller->addCSS($this->_path . 'views/css/menuTabIcon.css');
    }
    
    public function hookDisplayAdminProductsExtra($params)
    {
        return $this->display(_PS_MODULE_DIR_.'freshmail/views/templates/admin/alerts.tpl');
    }
    
    public function hookDisplayHeader($params)
    {
        return $this->hookHeader($params);
    }

    public function hookHeader($params)
    {
        $this->context->controller->addCSS(($this->_path).'views/css/front_left.css', 'all');
        $this->context->controller->addCSS(($this->_path).'views/css/front_footer.css', 'all');
        $this->context->controller->addJS($this->_path . 'views/js/alert.js');
    }

    public function hookDisplayCustomerAccountForm($params)
    {
        if(Configuration::get('FRESHMAIL_SIGNUP_ACTIVE') == 0) {
            return false;
        }
        $this->context->controller->addCSS(($this->_path).'views/css/front_checkbox_signup.css', 'all');
        $loadExstraFields = (int)Configuration::get('FRESHMAIL_SIGNUP_INCLUDE_FIELDS');

        $fields = Field::getFieldsForUserRegistrationForm();
        
        unset($fields['presta_newsletter']);
        unset($fields['presta_partners_newsletter']);
        
        $this->context->smarty->assign(array(
            'loadExstraFields' => $loadExstraFields,
            'fields' => $fields,
            'signUpSubmitLabel' => Configuration::get('FRESHMAIL_SIGNUP_LABEL'),
            'signUpSubmitAccept' => Configuration::get('FRESHMAIL_SIGNUP_ACTIVE'),
            'acceptEmailsPartnersActive' => Configuration::get('FRESHMAIL_ACCEPT_EMAILS_PARTNERS_ACTIVE'),
            'acceptEmailsPartnersActiveLabel' => Configuration::get('FRESHMAIL_ACCEPT_EMAILS_PARTNERS_LABEL'),
            'tpl_path', _PS_MODULE_DIR_.'views/templates/hook/form.tpl',
        ));
        
        return $this->display(__FILE__, 'customerAccountForm.tpl');
    }
    
    public function getContent()
    {
        $controller = 'FreshMailApi';
        $id_lang = Context::getContext()->language->id;
        $params = array('token' => Tools::getAdminTokenLite($controller));
        $url = Dispatcher::getInstance()->createUrl($controller, $id_lang, $params, true);
        Tools::redirectAdmin($url);
    }
    
    public function hookActionCustomerAccountAdd($params)
    {
        if(Configuration::get('FRESHMAIL_SIGNUP_ACTIVE') == 0) {
            return false;
        }
        $email = Tools::getValue('email');
        $tags = Tools::getValue('tag');
        
        if(!$tags['presta_newsletter']) {
            return false;
        }
        
        $hashList = Configuration::get('FRESHMAIL_SIGNUP_HASHLIST');
        $data = array(
            'email' => $email,
            'list'  => $hashList,
            'custom_fields' => array(
            ),
            'state'   => self::$SUBSCRIBER_STATUS_NO_ACTIVE,
            'confirm' => self::$CONFIRM_NO_ACTIVE
        );
        
        if(isset($tags['presta_newsletter'])) {
            $data['custom_fields']['presta_newsletter'] = $tags['presta_newsletter'];
        }
        
        if(isset($tags['presta_partners_newsletter'])) {
            $data['custom_fields']['presta_partners_newsletter'] = $tags['presta_partners_newsletter'];
        }
        
        $this->addNewSubscriber($data);
        
        return true;
    }
    
    private function _submit()
    {
        if (!empty($_POST)) {
            
            $this->displayConfirmation($this->l('Settings updated'));
            $list = Tools::getValue('list');
            
            $tags = Tools::getValue('tag');
            $email = isset($tags['email'])? $tags['email'] : $this->context->customer->email;
            
            $status = self::$SUBSCRIBER_STATUS_NO_ACTIVE;
            $confirm = self::$CONFIRM_NO_ACTIVE;
             
            if (!Validate::isEmail($email)) {    
                $this->context->smarty->assign(array(
                    'msgFm' => Configuration::get('FRESHMAIL_INVALID_EMAIL_ADDRESS_MESSAGE'),
                    'errorFm' => true
                ));
            } elseif(empty($tags)) {
                $this->context->smarty->assign(array(
                    'msgFm' => Configuration::get('FRESHMAIL_REQUIRED_FIELD_MISSING_MESSAGE'),
                    'errorFm' => true
                ));
            } else {
                $data = array(
                    'email' => $email,
                    'list'  => $list,
                    'custom_fields' => array(
                    ),
                    'state'   => self::$SUBSCRIBER_STATUS_NO_ACTIVE,
                    'confirm' => self::$CONFIRM_NO_ACTIVE
                );
                unset($tags['email']);
                
                if(!empty($tags)) {
                    foreach($tags as $k => $v) {
                        $data['custom_fields'][$k] = $v;
                    }
                }
                
                $this->addNewSubscriber($data);
            }
        }
    }
    
    public function addNewSubscriber($data = null) {
        if(empty($data)) {
            return false;
        }
        
        try {
            $api = new Api();
            $api->addSubscriber($data);
            
            $this->context->smarty->assign(array(
                'msgFm' => Configuration::get('FRESHMAIL_SUBMISSION_SUCCESS_MESSAGE'),
                'errorFm' => false
            ));
            
        } catch(\Exception $e) {
            
            if($e->getCode() == 1304) {
                $msg = Configuration::get('FRESHMAIL_ALREADY_SUBSCRIBED_MESSAGE');
            } else {
                $msg = Configuration::get('FRESHMAIL_SUBMISSION_FAILURE_MESSAGE');
            }

            $this->context->smarty->assign(array(
                'msgFm' => $msg,
                'errorFm' => true
            ));
        }
        
        return true;
    }
    
    public function hookDisplayTop($params) {
        $fields = FormRepository::getActiveFormByPosition(self::$TOP);
        
        $forms = array();
        foreach($fields as $k => $field) {
            $forms[$field['id_form']][] = $field;
        }
        
        if(count($forms) == 0) {
            return false;
        }
        
        foreach($forms as $id => $v) {
            if(isset($_POST['submit_freshmail_' . $id])) {
                $this->_submit();
            }
        }
        
        $pathCss = $this->_path.'view/css/front_top.css';
        $this->context->controller->addCSS($pathCss, 'all');
        $this->context->smarty->assign(array(
            'placement' => 'top',
            'forms' => $forms,
            'label_submit' => Configuration::get('FRESHMAIL_FORM_SUBSCRIBE_BUTTON')
        ));
        return $this->display(__FILE__, 'top.tpl');
    }
    
    public function hookDisplayLeftColumn($params)
    {
        $fields = FormRepository::getActiveFormByPosition(self::$LEFT_COLUMN);
        
        $forms = array();
        foreach($fields as $k => $field) {
            $forms[$field['id_form']][] = $field;
        }
        
        if(count($forms) == 0) {
            return false;
        }
        foreach($forms as $id => $v) {
            if(isset($_POST['submit_freshmail_' . $id])) {
                $this->_submit();
            }
        }
        
        $pathCss = $this->_path.'view/css/front_left.css';
        $this->context->controller->addCSS($pathCss, 'all');
        $this->context->smarty->assign(array(
            'placement' => 'left',
            'forms' => $forms,
            'label_submit' => Configuration::get('FRESHMAIL_FORM_SUBSCRIBE_BUTTON')
        ));
        return $this->display(__FILE__, 'left.tpl');
    }

    public function hookDisplayRightColumn($params)
    {
        $fields = FormRepository::getActiveFormByPosition(self::$RIGHT_COLUMN);
        
        $forms = array();
        foreach($fields as $k => $field) {
            $forms[$field['id_form']][] = $field;
        }
        
        if(count($forms) == 0) {
            return false;
        }
        
        foreach($forms as $id => $v) {
            if(isset($_POST['submit_freshmail_' . $id])) {
                $this->_submit();
            }
        }
        
        $pathCss = $this->_path.'view/css/front_right.css';
        $this->context->controller->addCSS($pathCss, 'all');
        $this->context->smarty->assign(array(
            'placement' => 'right',
            'forms' => $forms,
            'label_submit' => Configuration::get('FRESHMAIL_FORM_SUBSCRIBE_BUTTON')
        ));
        return $this->display(__FILE__, 'right.tpl');
    }

    public function hookDisplayFooter($params)
    {
       $fields = FormRepository::getActiveFormByPosition(self::$FOOTER);
        
        $forms = array();
        foreach($fields as $k => $field) {
            $forms[$field['id_form']][] = $field;
        }
        foreach($forms as $id => $v) {
            if(isset($_POST['submit_freshmail_' . $id])) {
                $this->_submit();
            }
        }
        
        $pathCss = $this->_path.'view/css/front_footer.css';
        $this->context->controller->addCSS($pathCss, 'all');
        $this->context->smarty->assign(array(
            'placement' => 'footer',
            'forms' => $forms,
            'label_submit' => Configuration::get('FRESHMAIL_FORM_SUBSCRIBE_BUTTON')
        ));
        return $this->display(__FILE__, 'footer.tpl');
    }
}